import Vue from 'vue'
import Router from 'vue-router'

Vue.use(Router)

/* Layout */
import Layout from '@/layout'

/**
 * Note: 路由配置项
 *
 * hidden: true                     // 当设置 true 的时候该路由不会再侧边栏出现 如401，login等页面，或者如一些编辑页面/edit/1
 * alwaysShow: true                 // 当你一个路由下面的 children 声明的路由大于1个时，自动会变成嵌套的模式--如组件页面
 *                                  // 只有一个时，会将那个子路由当做根路由显示在侧边栏--如引导页面
 *                                  // 若你想不管路由下面的 children 声明的个数都显示你的根路由
 *                                  // 你可以设置 alwaysShow: true，这样它就会忽略之前定义的规则，一直显示根路由
 * redirect: noRedirect             // 当设置 noRedirect 的时候该路由在面包屑导航中不可被点击
 * name:'router-name'               // 设定路由的名字，一定要填写不然使用<keep-alive>时会出现各种问题
 * query: '{"id": 1, "name": "ry"}' // 访问路由的默认传递参数
 * roles: ['admin', 'common']       // 访问路由的角色权限
 * permissions: ['a:a:a', 'b:b:b']  // 访问路由的菜单权限
 * meta : {
    noCache: true                   // 如果设置为true，则不会被 <keep-alive> 缓存(默认 false)
    title: 'title'                  // 设置该路由在侧边栏和面包屑中展示的名字
    icon: 'svg-name'                // 设置该路由的图标，对应路径src/assets/icons/svg
    breadcrumb: false               // 如果设置为false，则不会在breadcrumb面包屑中显示
    activeMenu: '/system/user'      // 当路由设置了该属性，则会高亮相对应的侧边栏。
  }
 */
// 测试权限
// 公共路由
export const constantRoutes = [
  {
    path: '/redirect',
    component: Layout,
    hidden: true,
    children: [
      {
        path: '/redirect/:path(.*)',
        component: () => import('@/views/redirect')
      }
    ]
  },
  {
    path: '/login',
    component: () => import('@/views/login'),
    hidden: true
  },
  {
    path: '/',
    component: () => import('@/views/login'),
    hidden: true
  },
  {
    path: '/home',
    component: () => import('@/views/home'),
    hidden: true
  },
  {
    path: '/register',
    component: () => import('@/views/register'),
    hidden: true
  },
  {
    path: '/404',
    component: () => import('@/views/error/404'),
    hidden: true
  },
  {
    path: '/401',
    component: () => import('@/views/error/401'),
    hidden: true
  },
  {
    path: '',
    component: Layout,
    redirect: 'index',
    meta: { title: '', icon: 'dashboard', affix: true, link: null },
    children: [
      {
        path: 'server',
        component: () => import('@/views/monitor/server/index.vue'),
        name: 'Index',
        meta: { title: '首页', icon: 'dashboard', affix: true, link: null }
      },
      // {
      //   path: 'homes',
      //   component: () => import('@/views/ceshi/homes.vue'),
      //   name: 'Index',
      //   meta: { title: '测试开发', icon: 'dashboard', affix: true, link: null }
      // }
      // {
      //         path: 'statistics',
      //         component: () => import('@/views/emergencies/statistics'),
      //         name: 'homes',
      //         meta: { title: '演习统计', icon: 'dashboard', affix: true },
      //       },
      //       {
      //         path: 'summarize',
      //         component: () => import('@/views/emergencies/summarize'),
      //         name: 'homes',
      //         meta: { title: '演习总结', icon: 'dashboard', affix: true },
      //       },
      // {
      //   path: '/document',
      //   component: () => import('@/views/system/document/index.vue'),
      //   name: 'document',
      //   meta: { title: '安全基础文档', icon: 'user', affix: true }
      // },
    ]

},
  //     {
  //       path: 'allocation',
  //       component: () => import('@/views/emergencies/allocation'),
  //       name: 'homes',
  //       meta: { title: '演习配置', icon: 'dashboard', affix: true },
  //     },
  //     {
  //       path: 'yearmanoeuvre',
  //       component: () => import('@/views/emergencies/yearmanoeuvre'),
  //       name: 'homes',
  //       meta: { title: '年度演习计划', icon: 'dashboard', affix: true },
  //     },
  //     {
  //       path: '/manoeuvre',
  //       component: () => import('@/views/emergencies/manoeuvre'),
  //       name: 'homes',
  //       meta: { title: '演习计划', icon: 'dashboard', affix: true },
  //     },
  //     {
  //       path: '/jobSignatureTemplate',
  //       component: () => import('@/views/jobSheet/jobSignatureTemplate.vue'),
  //       name: 'jobSignatureTemplate',
  //       meta: { title: '作业签字模板', icon: 'dashboard', affix: true },
  //     },
  //     {
  //       path: '/safetyPrecautionsTemplate',
  //       component: () => import('@/views/jobSheet/safetyPrecautionsTemplate.vue'),
  //       name: 'safetyPrecautionsTemplate',
  //       meta: { title: '安全措施模板', icon: 'dashboard', affix: true },
  //     },
  //     {
  //       path: '/jobList',
  //       component: () => import('@/views/jobSheet/jobList.vue'),
  //       name: 'jobList',
  //       meta: { title: '作业列表', icon: 'dashboard', affix: true },
  //     },
  //     {
  //       path: '/profession',
  //       component: () => import('@/views/contractor/profession.vue'),
  //       name: 'profession',
  //       meta: { title: '承包商工种配置', icon: 'dashboard', affix: true },
  //     },
  //     // 计划分类
  //     {
  //       path: '/planClass',
  //       component: () => import('@/views/planClass/planClass.vue'),
  //       name: 'planClass',
  //       meta: { title: '计划分类', icon: 'user', affix: true }
  //     },
  //     // 题库
  //     {
  //       path: '/questionBank',
  //       component: () => import('@/views/planClass/questionBank.vue'),
  //       name: 'questionBank',
  //       meta: { title: '题库', icon: 'user', affix: true }
  //     },
  //     // 课程
  //     {
  //       path: '/course',
  //       component: () => import('@/views/planClass/course.vue'),
  //       name: 'course',
  //       meta: { title: '课程', icon: 'user', affix: true }
  //     },
  //     // 设备设施管理
  //     {
  //       path: '/facility',
  //       component: () => import('@/views/facility/facility.vue'),
  //       name: 'facility',
  //       meta: { title: '设备设施', icon: 'user', affix: true }
  //     },
  //     // 风险数据管理
  //     {
  //       path: '/controlObject',
  //       component: () => import('@/views/risk/controlObject.vue'),
  //       name: 'controlObject',
  //       meta: { title: '管控对象', icon: 'user', affix: true }
  //     },
  //     {
  //       path: '/analyse',
  //       component: () => import('@/views/risk/analyse.vue'),
  //       name: 'analyse',
  //       meta: { title: '分析单元', icon: 'user', affix: true }
  //     },
  //     {
  //       path: '/riskEvent',
  //       component: () => import('@/views/risk/riskEvent.vue'),
  //       name: 'riskEvent',
  //       meta: { title: '风险事件', icon: 'user', affix: true }
  //     },
  //     {
  //       path: '/controlMeasure',
  //       component: () => import('@/views/risk/controlMeasure.vue'),
  //       name: 'controlMeasure',
  //       meta: { title: '风险管控措施', icon: 'user', affix: true }
  //     },
  //     {
  //       path: '/checkAsk',
  //       component: () => import('@/views/risk/checkAsk.vue'),
  //       name: 'checkAsk',
  //       meta: { title: '隐患排查要求', icon: 'user', affix: true }
  //     },
  //     {
  //       path: '/secure',
  //       component: () => import('@/views/facility/secure.vue'),
  //       name: 'secure',
  //       meta: { title: '安全基础文档', icon: 'user', affix: true }
  //     },
  //     {
  //       path: '/nameConfig',
  //       component: () => import('@/views/facility/nameConfig.vue'),
  //       name: 'nameConfig',
  //       meta: { title: '快捷命名配置', icon: 'user', affix: true }
  //     },
  //     {
  //       path: '/document',
  //       component: () => import('@/views/system/document/index.vue'),
  //       name: 'document',
  //       meta: { title: '安全基础文档', icon: 'user', affix: true }
  //     },
  //   ]
  // },
  // {
  //   path: '/trouble',
  //   component: Layout,
  //   redirect: 'trouble',
  //   meta: { title: '隐患管理', icon: 'dashboard', affix: true, link: null },
  //   children: [
  //     {
  //       path: '/trouble',
  //       component: () => import('@/views/trouble/index'),
  //       name: 'homes',
  //       meta: { title: '隐患列表', icon: 'dashboard', affix: true },
  //     },
  //     {
  //       path: '/allocation',
  //       component: () => import('@/views/emergencies/allocation'),
  //       name: 'homes',
  //       meta: { title: '演习配置', icon: 'dashboard', affix: true },
  //     },
  //   ]
  // },
  // {
  //   path: '/contractor',
  //   component: Layout,
  //   redirect: 'trouble',
  //   meta: { title: '郭浩', icon: 'dashboard', affix: true, link: null },
  //   children: [
  //     {
  //       path: '/contractor',
  //       component: () => import('@/views/contractor/contractor.vue'),
  //       name: 'contractor',
  //       meta: { title: '承包商配置', icon: 'dashboard', affix: true },
  //     },
  //     {
  //       path: '/contractorUser',
  //       component: () => import('@/views/contractor/contractorUser.vue'),
  //       name: 'contractorUser',
  //       meta: { title: '承包商用户配置', icon: 'dashboard', affix: true },
  //     },
  //   ]
  // },
  {
    path: '/user',
    component: Layout,
    hidden: true,
    redirect: 'noredirect',
    children: [
      {
        path: 'profile',
        component: () => import('@/views/system/user/profile/index'),
        name: 'Profile',
        meta: { title: '个人中心', icon: 'user' }
      }
    ]
  }
]

// 动态路由，基于用户权限动态去加载
export const dynamicRoutes = [
  {
    path: '/system/user-auth',
    component: Layout,
    hidden: true,
    permissions: ['system:user:edit'],
    children: [
      {
        path: 'role/:userId(\\d+)',
        component: () => import('@/views/system/user/authRole'),
        name: 'AuthRole',
        meta: { title: '分配角色', activeMenu: '/system/user' }
      }
    ]
  },
  {
    path: '/system/role-auth',
    component: Layout,
    hidden: true,
    permissions: ['system:role:edit'],
    children: [
      {
        path: 'user/:roleId(\\d+)',
        component: () => import('@/views/system/role/authUser'),
        name: 'AuthUser',
        meta: { title: '分配用户', activeMenu: '/system/role' }
      }
    ]
  },
  {
    path: '/system/dict-data',
    component: Layout,
    hidden: true,
    permissions: ['system:dict:list'],
    children: [
      {
        path: 'index/:dictId(\\d+)',
        component: () => import('@/views/system/dict/data'),
        name: 'Data',
        meta: { title: '字典数据', activeMenu: '/system/dict' }
      }
    ]
  },
  {
    path: '/monitor/job-log',
    component: Layout,
    hidden: true,
    permissions: ['monitor:job:list'],
    children: [
      {
        path: 'index/:jobId(\\d+)',
        component: () => import('@/views/monitor/job/log'),
        name: 'JobLog',
        meta: { title: '调度日志', activeMenu: '/monitor/job' }
      }
    ]
  },
  {
    path: '/tool/gen-edit',
    component: Layout,
    hidden: true,
    permissions: ['tool:gen:edit'],
    children: [
      {
        path: 'index/:tableId(\\d+)',
        component: () => import('@/views/tool/gen/editTable'),
        name: 'GenEdit',
        meta: { title: '修改生成配置', activeMenu: '/tool/gen' }
      }
    ]
  }
]

// 防止连续点击多次路由报错
let routerPush = Router.prototype.push;
let routerReplace = Router.prototype.replace;
// push
Router.prototype.push = function push(location) {
  return routerPush.call(this, location).catch(err => err)
}
// replace
Router.prototype.replace = function push(location) {
  return routerReplace.call(this, location).catch(err => err)
}

export default new Router({
  mode: 'history', // 去掉url中的#
  scrollBehavior: () => ({ y: 0 }),
  routes: constantRoutes
})
